/* file jisp.c */
/*
 * kf linux jabber client
 * ----------------------
 *
 * Copyright (C) 2003-2004 Przemysaw Sitek <psitek@rams.pl> 
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <glib.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <libxml/tree.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>	/* for mkdir () */
#include <sys/types.h>	/*              */
#include "kf.h"
#include "jisp.h"

static KfJispIcon str2jispicon (const gchar *name);
static void kf_jisp_update_preview (KfJisp *jisp);


static gchar *names[] = {"status/online", "status/chat", "status/away", "status/xa", "status/dnd", "status/invisible", "status/offline", NULL};



KfJisp *kf_jisp_new (void) {
	KfJisp *jisp = g_new0 (KfJisp, 1);
	jisp->ref_count = 1;

	g_print ("kf_jisp_new ()\n");

	return jisp;
}


KfJisp *kf_jisp_ref (KfJisp *jisp) {
	jisp->ref_count++;
	return jisp;
}


void kf_jisp_unref (KfJisp *jisp) {
	if (--(jisp->ref_count) == 0)
		kf_jisp_free (jisp);
}


void kf_jisp_free (KfJisp *jisp) {
	int i;

	foo_debug ("Freeing JISP %s (%s)\n", jisp->name, jisp->filename);

	for (i = 0; i < JISP_N_ICONS; i++)
		if (jisp->icons[i])
			gdk_pixbuf_unref (jisp->icons[i]);
	
	gdk_pixbuf_unref (jisp->preview);
	g_free (jisp->name);
	g_free (jisp->desc);
	g_free (jisp->authors);
	g_free (jisp->filename);
	g_free (jisp);
}


KfJisp *kf_jisp_new_from_file (const gchar *file) {
	KfJisp *jisp;
	xmlDocPtr doc;
	xmlNodePtr node;
	gchar *path;

	foo_debug ("kf_jisp_new_from_file (%s)\n", file);

	jisp = kf_jisp_new ();
	jisp->filename = g_strdup (file);

	doc = xmlParseFile (file);
	if (doc == NULL) {
		xmlFreeDoc (doc);
		return NULL;
	}

	node = xmlDocGetRootElement (doc);
	if (node == NULL) {
		xmlFreeDoc (doc);
		return NULL;
	}
	if (xmlStrcmp (node->name, (const xmlChar *) "icondef")) {
		xmlFreeDoc (doc);
		return NULL;
	}

	path = g_path_get_dirname (file);

	for (node = node->xmlChildrenNode; node; node = node->next) {

//		foo_debug (" * processing <%s>\n", node->name);
		if (node->type != XML_ELEMENT_NODE) {
			continue;
		}
		if (xmlStrcmp (node->name, (const xmlChar *) "meta") == 0) {
			/* Meta-information about package */
			xmlNodePtr child;

			for (child = node->xmlChildrenNode; child; child = child->next) {
//				foo_debug ("   * processing <%s>\n", child->name);
				xmlChar *value = xmlNodeGetContent (child);
				if (xmlStrcmp (child->name, (const xmlChar *) "name") == 0) {
					jisp->name = g_strdup (value);
//				} else if (xmlStrcmp (child->name, (const xmlChar *) "") == 0) {
//					jisp-> = g_strdup (value);
				} else if (xmlStrcmp (child->name, (const xmlChar *) "description") == 0) {
					jisp->desc = g_strdup (value);
				} else if (xmlStrcmp (child->name, (const xmlChar *) "author") == 0) {
					jisp->authors = g_strdup (value);
				}
				xmlFree (value);
			}
			
		} else if (xmlStrcmp (node->name, (const xmlChar *) "icon") == 0) {
			/* Icon */
			xmlChar *name = NULL;
			xmlChar *filename = NULL;
			xmlNodePtr child;
			
			for (child = node->xmlChildrenNode; child; child = child->next) {
//				foo_debug ("   * processing <%s>\n", child->name);
				if (xmlStrcmp (child->name, (const xmlChar *) "x") == 0) {
					xmlChar *key;
					key = xmlGetProp (node, "xmlns");
				//	if (xmlStrcmp (key, "name") == 0) {
//						foo_debug ("fetching name...\n");
						name = xmlNodeGetContent (child);
				//	}
					xmlFree (key);
				} else if (xmlStrcmp (child->name, (const xmlChar *) "object") == 0) {
					filename = xmlNodeGetContent (child);
				}
			}
//			foo_debug ("        icon name='%s' file='%s'\n", name, filename);
			if (name && filename) {
				KfJispIcon i;

				/* get index */
				i = str2jispicon (name);
//				foo_debug ("str2jispicon (%s) = %d\n", name, i);
				if (i >= 0 && i < JISP_N_ICONS) {
					/* Load image */
					GdkPixbuf *image;
					GError *error = NULL;
					gchar *img_path = g_strdup_printf ("%s/%s", path, filename);
					foo_debug ("Trying to load image %s... ", img_path);
					image = gdk_pixbuf_new_from_file (img_path, &error);

					if (image) {
						jisp->icons[i] = image;
						foo_debug ("loaded\n");
					} else {
						foo_debug ("failed to load\n");
					}
				}
			}
			xmlFree (name);
			xmlFree (filename);
		}
	}
	kf_jisp_update_preview (jisp);

	g_free (path);
	xmlFreeDoc (doc);
	xmlCleanupParser ();

	return jisp;
}


KfJisp *kf_jisp_install (const gchar *zipfile) {
	FILE *p;	/* Pipe */
	GSList *files = NULL;	/* Files contained in archive */
	gchar *file;	/* quoted zipfile */
	gchar *cmd;	/* Command */
	KfJisp *jisp;
	gchar buffer[4096];
	const gchar *root = NULL;	/* Root path of zipfile */
	const gchar *icondef = NULL;
	gboolean good_jisp = TRUE;
	const gchar *path;
	gchar *icondef_absolute;
	
	foo_debug ("kf_jisp_install (%s)\n", zipfile);
	
	if (! g_file_test (zipfile, G_FILE_TEST_EXISTS))
		return NULL;

	file = g_shell_quote (zipfile);
	cmd = g_strdup_printf ("zipinfo -1 %s", file);
	p = popen (cmd, "r");
	g_free (cmd);

	if (p == NULL) {
		g_free (file);
		return NULL;
	}
	
	while (fgets (buffer, 4096, p)) {
		gchar *line;
		/* Get rid of '\n' at the end of line */
		gchar *c = buffer;
		while (*c != '\n')
			c++;
		*c = '\0';

		foo_debug (" * file %s\n", buffer);

		line = g_strdup (buffer);
		if (root == NULL) {
			root = line;
		} else {
			/* Check if there is more than one directory */
			if (strstr (line, root) != line)
				good_jisp = FALSE;
			/* Check if we have icondef.xml in this
			 * archive */
			if (strstr (line, "icondef.xml") != NULL)
				icondef = line;
		}
		files = g_slist_prepend (files, line);
	}
	pclose (p);
	
	if (files == NULL) {
		g_free (file);
		return NULL;
	}

	foo_debug (" * good_jisp=%d icondef='%s'\n", good_jisp, icondef);
	if (! good_jisp || ! icondef) {
		/* It's no good file :-/ */
		g_free (file);
		return NULL;
	}
	foo_debug ("X");

	path = kf_config_file ("icons");
	if (! g_file_test (path, G_FILE_TEST_IS_DIR)) {
		foo_debug (" * Creating %s\n", path);
		mkdir (path, 00753);
	}

	/* Extract files */
	cmd = g_strdup_printf ("unzip -d %s %s", path, file);
	foo_debug (" * running '%s'\n", cmd);
	system (cmd);
	g_free (cmd);

	icondef_absolute = g_strdup_printf ("%s/%s", path, icondef);
	jisp = kf_jisp_new_from_file (icondef_absolute);
	g_free (icondef_absolute);

	if (jisp == NULL) {
		/* Get rid of useless files */
		GSList *tmp;

		for (tmp = files; tmp; tmp=tmp->next) {
			gchar *absolute = g_strdup_printf ("%s/%s", path, (gchar *) tmp->data);
			remove (absolute);
			g_free (absolute);
			g_free (tmp->data);
		}
	} else {
		GSList *tmp;

		for (tmp = files; tmp; tmp=tmp->next) {
			g_free (tmp->data);
		}
	}
	g_slist_free (files);

	g_free (file);
	return jisp;
}

/* Convert name in icondef.xml file to KfJispIcon */
static KfJispIcon str2jispicon (const gchar *name) {
	gint i = 0;
	while (names[i] != NULL) {
		if (strcmp (name, names[i]) == 0)
			break;
		i++;
	}
	if (names[i] == NULL)
		return -1;
	else
		return i;
}


static void kf_jisp_update_preview (KfJisp *jisp) {
	gint width = 0;
	gint height = 0;
	GdkPixbuf *preview;
	gint i;
	gint x = 0;

	foo_debug ("kf_jisp_update_preview ()\n");

	/* Compute preview size */
	for (i = 0; i < KF_ICON_UNAVAILABLE; i++) {
		if (jisp->icons[i]) {
			width += gdk_pixbuf_get_width (jisp->icons[i]) + 2;
			height = MAX (height, gdk_pixbuf_get_height (jisp->icons[i]));
		}
	}

	preview = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
			TRUE, 8,
			width, height);
	if (preview == NULL) {
		foo_debug ("unable to create preview\n");
		return;
	}
	/* Fill everything with transparent color;-) */
	gdk_pixbuf_fill (preview, 0x00000000);
	for (i = 0; i < KF_ICON_UNAVAILABLE; i++) {
		if (jisp->icons[i]) {
			gdk_pixbuf_copy_area (jisp->icons[i],
				0, 0,
				gdk_pixbuf_get_width (jisp->icons[i]),
				gdk_pixbuf_get_height (jisp->icons[i]),
				preview,
				x, 0);
			x += gdk_pixbuf_get_width (jisp->icons[i]) + 2;
		}
	}
	if (jisp->preview)
		gdk_pixbuf_unref (jisp->preview);
	jisp->preview = preview;
	foo_debug ("kf_jisp_update_preview... done\n");
}
